//
// time.c
//
// Time routines
//
// Copyright (C) 2002 Michael Ringgaard. All rights reserved.
// Copyright (C) 2005 Jorgen Birkler
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.  
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.  
// 3. Neither the name of the project nor the names of its contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
// 


// AVR clib changes (mainly to reduce size, approx half original code):
//
// - changed to uint16_t from long where possible
// - replaced macros by static functions to keep size down
// - replaced summing of days from beginning of year in a array
// - Approximates year overflow with MAX_YEARS_APPROX instead of 
//   calculating exactly to the bit near of MAX_LONG
// - By buffer one year before overflow the seconds from month,day,hour,min,sec
//   can be safely added without checking for overflow.
//

#include <timeh_private.h>
#include <avr/interrupt.h>


#define CRITICAL_SECTION_ENTER() { uint8_t saved_sreg = SREG; cli(); {

time_t current_time __attribute__ ((section (".noinit")));

#define CRITICAL_SECTION_EXIT() } SREG = saved_sreg; }

static time_t get_current_time(void)
{
	time_t val;
	CRITICAL_SECTION_ENTER()
	val = current_time;
	CRITICAL_SECTION_EXIT()
	return val;
}

clock_t clock(void)
{
	return (clock_t)get_current_time();
}

time_t time(time_t *timeptr)
{
	return (*timeptr = get_current_time());
}

void rtc_update(time_t delta)
{
	CRITICAL_SECTION_ENTER()
	current_time += delta;
	CRITICAL_SECTION_EXIT()
}

void rtc_set_time(time_t time)
{
	CRITICAL_SECTION_ENTER()
	current_time = time;
	CRITICAL_SECTION_EXIT()
}




