//
// time.c
//
// Time routines
//
// Copyright (C) 2002 Michael Ringgaard. All rights reserved.
// Copyright (C) 2005 Jorgen Birkler
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.  
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.  
// 3. Neither the name of the project nor the names of its contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
// 

// AVR clib changes (mainly to reduce size, approx half original code):
//
// - changed to uint16_t from long where possible
// - replaced macros by functions to keep size down
// - replaced summing of days from beginning of year in a array
// - Approximates year overflow with MAX_YEARS_APPROX instead of 
//   calculating exactly to the bit near of MAX_LONG
// - By buffer one year before overflow the seconds from month,day,hour,min,sec
//   can be safely added without checking for overflow.
//

#include <timeh_private.h>
#include <avr/pgmspace.h>

int _daylight = 0;              // Non-zero if daylight savings time is used
long _dstbias = 0;              // Offset for Daylight Saving Time
long _timezone = 0;             // Difference in seconds between GMT and local time
//char *_tzname[2] = {"GMT", "GMT"};  // Standard/daylight savings time zone names

uint8_t is_leap_year(uint16_t year)
{
	return (!((year) % 4) && (((year) % 100) || !((year) % 400)));
}

uint8_t days_in_month(uint16_t year, uint8_t month)
{
	static const prog_uint8_t _year_table[2][12] = 
	{
	  {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	  {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
	};
	return pgm_read_byte_near(&_year_table[is_leap_year(year)][month]);
}

uint16_t days_in_year(uint16_t year)
{
	return is_leap_year(year) ? 366 : 365;
}




