//
// timeh_private.h
//
// Time routines
//
// Copyright (C) 2005 Jorgen Birkler
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.  
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.  
// 3. Neither the name of the project nor the names of its contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
// 


// AVR clib changes (mainly to reduce size, approx half original code):
//
// - changed to uint16_t from long where possible
// - replaced macros by static functions to keep size down
// - replaced summing of days from beginning of year in a array
// - Approximates year overflow with MAX_YEARS_APPROX instead of 
//   calculating exactly to the bit near of MAX_LONG
// - By buffer one year before overflow the seconds from month,day,hour,min,sec
//   can be safely added without checking for overflow.
//

#include <time.h>
#include <inttypes.h>
#include <avr/pgmspace.h>

#define YEAR0                   1900
#define EPOCH_YR                1970
#define SECS_DAY                (24L * 60L * 60L)
#define FIRSTSUNDAY(timp)       (((timp)->tm_yday - (timp)->tm_wday + 420) % 7)
#define FIRSTDAYOF(timp)        (((timp)->tm_wday - (timp)->tm_yday + 420) % 7)
#define TIME_MAX                2147483647L
#define MAX_YEARS_APPROX        (TIME_MAX / (366L * SECS_DAY))
#define END_EPOCH_YR            (EPOCH_YR+MAX_YEARS_APPROX)

uint8_t is_leap_year(uint16_t year);

uint8_t days_in_month(uint16_t year, uint8_t month);

uint16_t days_in_year(uint16_t year);



